'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var QueryBuilder = function () {
    function QueryBuilder() {
        (0, _classCallCheck3.default)(this, QueryBuilder);

        this.query = '';
        this.relations = [];
        this.sorts = [];
        this.fields = [];
        this.limitRows = null;
        this.fromDate = null;
        this.toDate = null;
        this.selectDistinct = false;
        this.wheres = [];
    }

    (0, _createClass3.default)(QueryBuilder, [{
        key: 'with',
        value: function _with() {
            for (var _len = arguments.length, resourceName = Array(_len), _key = 0; _key < _len; _key++) {
                resourceName[_key] = arguments[_key];
            }

            if (!this.relations[resourceName]) {
                var _relations;

                (_relations = this.relations).push.apply(_relations, resourceName);
            }
        }
    }, {
        key: 'limit',
        value: function limit(_limit, offset) {
            this.limitRows = { limit: _limit, offset: offset };
        }
    }, {
        key: 'paginate',
        value: function paginate() {
            var perPage = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
            var pageNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

            this.limit(perPage, pageNumber * perPage - perPage);
        }
    }, {
        key: 'orderBy',
        value: function orderBy(column) {
            var direction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'asc';

            direction = direction.toLowerCase();
            if (!['asc', 'desc'].includes(direction)) {
                throw new Error('Sarale: Invalid sort direction: "' + direction + '". Allowed only "asc" or "desc" (case insensitive).');
            }

            this.sorts.push({ column: column, direction: direction });
        }
    }, {
        key: 'where',
        value: function where(key, operator, value) {
            if (_lodash2.default.isNull(value)) {
                value = operator;
                operator = '=';
            }

            this.wheres.push({ key: key, operator: operator, value: value });
        }
    }, {
        key: 'select',
        value: function select() {
            for (var _len2 = arguments.length, fields = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                fields[_key2] = arguments[_key2];
            }

            this.fields = fields;
        }
    }, {
        key: 'distinct',
        value: function distinct() {
            var bool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            this.selectDistinct = bool;
        }
    }, {
        key: 'from',
        value: function from(date) {
            this.from = date;
        }
    }, {
        key: 'to',
        value: function to(date) {
            this.to = date;
        }
    }, {
        key: 'getQuery',
        value: function getQuery() {
            this.appendIncludes();
            this.appendLimit();
            this.appendSort();
            this.appendFromTo();
            this.appendFields();
            this.appendWheres();
            this.appendDistinct();

            if (this.query.length) {
                this.query = '?' + this.query;
            }

            return this.query;
        }
    }, {
        key: 'appendDistinct',
        value: function appendDistinct() {
            if (this.selectDistinct) {
                this.appendQuery(_config.REST_CONFIG.request_keywords.selectDistinct + '=' + this.selectDistinct);
            }
        }
    }, {
        key: 'appendFromTo',
        value: function appendFromTo() {
            if (this.fromDate) {
                this.appendQuery(_config.REST_CONFIG.request_keywords.from + '=' + this.fromDate);
            }

            if (this.toDate) {
                this.appendQuery(_config.REST_CONFIG.request_keywords.to + '=' + this.toDate);
            }
        }
    }, {
        key: 'appendWheres',
        value: function appendWheres() {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this.wheres[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var where = _step.value;

                    this.appendQuery(_config.REST_CONFIG.request_keywords.where + '[]=' + where.key + ',' + where.operator + ',' + where.value);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
    }, {
        key: 'appendIncludes',
        value: function appendIncludes() {
            if (this.relations.length) {
                this.appendQuery(_config.REST_CONFIG.request_keywords.load_relations + '=' + this.relations.join(','));
            }
        }
    }, {
        key: 'appendFields',
        value: function appendFields() {
            if (this.fields.length) {
                this.appendQuery(_config.REST_CONFIG.request_keywords.select_fields + '=' + this.fields.join(','));
            }
        }
    }, {
        key: 'appendSort',
        value: function appendSort() {
            if (this.sorts.length) {
                var fieldsArray = this.sorts.map(function (_ref) {
                    var column = _ref.column,
                        direction = _ref.direction;
                    return (direction === 'desc' ? '-' : '') + column;
                });
                this.appendQuery(_config.REST_CONFIG.request_keywords.order_by + '=' + fieldsArray.join(','));
            }
        }
    }, {
        key: 'appendLimit',
        value: function appendLimit() {
            if (this.limitRows) {
                this.appendQuery('limit=' + this.limitRows.limit + '&offset=' + this.limitRows.offset);
            }
        }
    }, {
        key: 'appendQuery',
        value: function appendQuery(append) {
            if (this.query.length) {
                append = '&' + append;
            }

            this.query += append;
        }
    }]);
    return QueryBuilder;
}();

exports.default = QueryBuilder;